/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_GATHERIMPL_FORWARD_KERNEL_H_
#define AIDGE_CPU_OPERATOR_GATHERIMPL_FORWARD_KERNEL_H_

#include "aidge/utils/Registrar.hpp"
#include <cstddef>
#include <cmath>
#include "aidge/data/Data.hpp"
#include "aidge/utils/Types.h"

#include "aidge/backend/cpu/operator/GatherImpl.hpp"

namespace Aidge {
template <class I, class O>
void GatherImpl_cpu_forward_kernel(std::size_t axisIdx, const std::vector<DimSize_t>& inputDims, const std::vector<DimSize_t>& indicesDims, const void* input_, const void* indexes_, void* output_)
{
    const I* input = static_cast<const I*>(input_);
    const int* indexes = static_cast<const int*>(indexes_);
    O* output = static_cast<O*>(output_);

    std::size_t postAxisElems = 1;
    for (std::size_t i = axisIdx + 1; i < inputDims.size(); ++i) {
        postAxisElems *= inputDims[i];
    }
    std::size_t preAxisElems = 1;
    for (std::size_t i = 0; i < axisIdx; ++i) {
        preAxisElems *= inputDims[i];
    }

    for (std::size_t i=0; i<preAxisElems; ++i)
    {
        for(std::size_t idxRow=0; idxRow<indicesDims[0]; ++idxRow)
        {
            for(std::size_t idxCol=0; idxCol<indicesDims[1]; ++idxCol)
            {
                std::size_t idx = indexes[indicesDims[1] *  idxRow + idxCol];
                const I* startPtr = std::next(input, i * postAxisElems * inputDims[axisIdx] + idx * postAxisElems);
                std::copy_n(startPtr, postAxisElems, output);
                output += postAxisElems;
            }
        }
    }
}

namespace {
static Registrar<GatherImplForward_cpu> registrarGatherImplForward_cpu_Float32(
        {DataType::Float32, DataType::Float32}, Aidge::GatherImpl_cpu_forward_kernel<float, float>);
static Registrar<GatherImplForward_cpu> registrarGatherImplForward_cpu_Int32(
        {DataType::Int32, DataType::Int32}, Aidge::GatherImpl_cpu_forward_kernel<int, int>);
static Registrar<GatherImplForward_cpu> registrarGatherImplForward_cpu_Float64(
        {DataType::Float64, DataType::Float64}, Aidge::GatherImpl_cpu_forward_kernel<double, double>);
}  // namespace
}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_GATHERIMPL_FORWARD_KERNEL_H_ */
